#!/bin/bash
#
# Copyright 2005 Nexenta Systems, Inc.  All rights reserved.
# Use is subject to license terms.
#
# Re-Upload .deb packages to repository
#

DIST=hardy-unstable
NEXENTA_LOCK_HELD=0
OUTPUT_EMAIL="devnull@nexenta.com"
KEEP=no
SEND_EMAIL=no
UPLOADED_FILES="files.uploaded"

trap "aborted" INT

fatal() {
        echo "$@"
        exit 1
}

cleanup() {
	if [ $NEXENTA_LOCK_HELD -ne 0 ]; then
		rmdir $NEXENTA_DEBLOCK >/dev/null 2>&1
	fi
}

aborted() {
	echo
	echo "Aborted."
	cleanup
	exit 1
}

usage() {
	echo "ndput <path-to-repository> [-h|--help] [-p package] -[kn]"
	echo "  -h         display usage screen"
	echo "  -p pkg     specify package or set name"
	echo "  -d distro  specify distro name, i.e. hardy-testing..."
	echo "  -k         do not remove packages after successfull upload"
	echo "  -e         send e-mail notification"
	exit 1
}

test "x$1" != x && REPOLOC="$1"
shift
while test "x$1" != x; do
	test "x$1" != x-h -a \
	     "x$1" != x--help -a \
	     "x$1" != x-k -a \
	     "x$1" != x-e -a \
	     "x$1" != x-d -a \
	     "x$1" != x--distro -a \
	     "x$1" != x-p -a \
	     "x$1" != x--package \
		&& usage
	test "x$1" = x-h -o "x$1" = x--help && usage
	if test "x$1" = "x-p" -o "x$1" = x--package; then PACKAGE=$2; shift; shift; fi
	if test "x$1" = "x-k"; then KEEP=yes; shift; fi
	if test "x$1" = "x-e"; then SEND_EMAIL=yes; shift; fi
	if test "x$1" = "x-d" -o "x$1" = x--distro; then DIST=$2; shift; shift; fi
done

test "x$REPOLOC" = x && usage

NEXENTA_DEBLOCK=$REPOLOC/dists/debarchiver.lock
NEXENTA_INCOMING=$REPOLOC/incoming

test ! -x /usr/bin/ndrm && fatal "missing /usr/bin/ndrm"
test ! -x /usr/bin/ndlist && fatal "missing /usr/bin/ndlist"

if test "x$PACKAGE" = x -a "x$(ls *.changes 2>/dev/null|wc -l)" = "x1"; then
	test -f ./*.changes && PACKAGE=$(echo *.changes|awk -F_ '{print $1}')
	test ! -f ./${PACKAGE}_*.changes && fatal "missing ${PACKAGE}_*.changes"
fi

if test "x$PACKAGE" = x; then
	echo "You must specify package or package set name. Use -p option."
	exit 1
fi

echo -n "Trying to obtain '$REPOLOC' repository lock ..."
while true; do
	mkdir $NEXENTA_DEBLOCK >/dev/null 2>&1
	if [ $? -ne 0 ]; then
		echo -n "."
		sleep 5
		continue
	fi
	echo "success!"
	NEXENTA_LOCK_HELD=1	
	break
done

# To account for stray files that have been uploaded, only
# put files related to changes
echo > $UPLOADED_FILES
for changes in *.changes; do

	if [ ! -r $changes ]; then
		continue
	fi

	# Remove old stuff..
	pkg=$(echo $changes|awk -F_ '{print $1}')
	ndrm $REPOLOC $pkg -n -y -d $DIST

	source_files=`awk '
		BEGIN {p=0;}
		/^[A-Za-z]/ {p=0}
		/^Files:/ {p=1}
		/^ / {if( p == 1 ) { print $5; } }
		' $changes;echo $changes`

	# Move files into the incoming dir for automatic processing
	if test "x$KEEP" = xno; then
		mv $source_files $NEXENTA_INCOMING/$DIST
	else
		cp $source_files $NEXENTA_INCOMING/$DIST
	fi

	echo >> $UPLOADED_FILES
	echo "$pkg:" >> $UPLOADED_FILES
	echo $source_files|sed -e "s/ /\n/g" >> $UPLOADED_FILES

done

rm -rf /var/cache/debarchiver/cache.db
/usr/bin/debarchiver --dl 1 --ql 0 --gpgkey Nexenta -o -x -i $REPOLOC/incoming
if test $? != 0; then
	cleanup
	echo "Repository '$REPOLOC': package(s) NOT uploaded."
	exit 1
fi

cleanup
echo "Repository '$REPOLOC': package(s) uploaded."

if test "x$SEND_EMAIL" = xno; then
	rm -f $UPLOADED_FILES
	exit 0
fi

# Email the record of the package
/usr/local/bin/changes-email "Accepted Package: $PACKAGE" $OUTPUT_EMAIL <<EOF
Accepted package(s) to $NEXENTA_INCOMING/$DIST

$(cat $UPLOADED_FILES)
EOF

rm -f $UPLOADED_FILES

# if e-mail not delivered still consider its a success...
exit 0
