#!/bin/bash

##############################################################################
#                      C O N F I G U R A T I O N
##############################################################################

ARCH=solaris-i386
DISTRO="hardy-unstable"
COMPONENTS="main non-free contrib"

##############################################################################

changes_files () {
	perl -le '
	$val="";
	while (<STDIN>) {
		chomp;
		last if (/^Files:.*$/);
	}
	while (<STDIN>) {
		if (/^ ([a-fA-F0-9]+) ([0-9]+) (.*) (.*) (.*)$/) {
			$val = "$val $5";
			next;
		}
		last;
	}
	printf ("%s", $val);
	' "$@"
}

changes_getfield () {
	perl -le '
	$field = lc(shift @ARGV);
	while (<STDIN>) {
		if (/^([^:]*:)\s*(.*)$/) {
			$f = lc($1); $v = $2;
			if ($f eq $field) { $val = $v; last; }
		}
	}
	printf ("%s", $val);
	' "$@"
}

all_changes() {
	echo "$REPOLOC/dists/$DISTRO/installed/$psource""_*""_$ARCH"".changes"
}

latest_changes() {
	echo $(ls $(all_changes)|grep $REPOLOC|sort|tail -1)
}

ndlist() {
	local changes=$(all_changes)
	if ! ls $changes 1>/dev/null 2>&1; then
		echo "$psource: not found"
		return 1
	fi
	echo $(changes_files < $(latest_changes)) | tr ' ' '\n'
	return 0
}

ndget() {
	local changes=$(all_changes)
	if ! ls $changes 1>/dev/null 2>&1; then
		echo "ndget: $psource: not found"
		return 1
	fi
	local vers=$(changes_getfield Version: < $(latest_changes))
	psourcedir=$psource-$(echo $vers | sed -e "s/.*:\(.*\)/\1/")
	rm -rf $psourcedir; mkdir $psourcedir
	for f in $(ndlist); do
		find $REPOLOC/dists/$DISTRO -name $f -exec cp {} $psourcedir/ \;
	done
	cp -a $changes $psourcedir/
	return 0
}

generic_Yn_ask() {
	test "x$allyes" != x && return
	while true; do
		echo -n "$1 [Y/n] "; read ans
		test x$ans = x -o x$ans = xy -o x$ans = xY && return 0
		test x$ans = xn -o x$ans = xN && return 1
	done
}

ndrm() {
	local changes=$(all_changes)
	if ! ls $changes 1>/dev/null 2>&1; then
		echo "ndrm: $psource: not found"
		return 1
	fi
	generic_Yn_ask "Are you sure you want to remove $psource from '$REPOLOC/dists/$DISTRO'?" || return 1
	for f in $(ndlist); do
		echo "Removing $REPOLOC/dists/$DISTRO $f ..."
		find $REPOLOC/dists/$DISTRO -name $f -exec rm -f {} \;
	done
	rm -f $changes
	if test "x$nodebarch" = x; then
		/usr/bin/debarchiver --dl 1 --ql 0 --gpgkey Nexenta -o -x -i $REPOLOC/incoming || return 1
	fi
	return 0
}

usage() {
	echo "Usage: $mode <path-to-repository> <source package> [-h|--help|-n|-y] [-d distro]"
	exit 0
}

mode=`basename $0`
test "x$1" != x && REPOLOC="$1"
shift
test "x$1" != x && psource="$1"
shift
test "x$REPOLOC" = x -o "x$psource" = x && usage
while test "x$1" != x; do
	test "x$1" != x-h -a \
	     "x$1" != x--help -a \
	     "x$1" != x-n -a \
	     "x$1" != x-d -a \
	     "x$1" != x-y \
		&& usage
	test "x$1" = x-h -o "x$1" = x--help && usage
	if test "x$1" = x-n; then nodebarch=yes; shift; fi
	if test "x$1" = x-y; then allyes=yes; shift; fi
	if test "x$1" = "x-d"; then DISTRO=$2; shift; shift; fi
done

if test $UID != 0 -a $mode != ndlist; then
	echo "You must become root if you want to use nexenta_dput!"
	exit 1
fi

if test $mode = ndlist; then
	ndlist
elif test $mode = ndget; then
	ndget
elif test $mode = ndrm; then
	ndrm
fi
